@@ -5,12 +5,12 @@ use rusttype::Font;
 use std::ops::Range;
 
 #[derive(Debug)]
-pub(super) enum Element<'a> {
-    Img(Option<Img<'a>>),
-    Text(Option<Text<'a>>),
+pub(super) enum Element {
+    Img(Option<Img>),
+    Text(Option<Text>),
 }
 
-impl<'a> Element<'a> {
+impl Element {
     pub(super) fn is_absolute(&self) -> bool {
         match self {
             Element::Img(Some(img)) => matches!(img.style.position, Position::Absolute),
@@ -48,14 +48,14 @@ impl Rect {
 }
 
 #[derive(Debug)]
-pub struct Img<'a> {
+pub struct Img {
     pub(super) buf: ImageBuffer<Rgba<u8>, Vec<u8>>,
     pub(super) rect: Rect,
-    pub(super) style: Style<'a>,
+    pub(super) style: Style,
 }
 
-impl<'a> Img<'a> {
-    pub fn new(buf: ImageBuffer<Rgba<u8>, Vec<u8>>, rect: Rect, style: Style<'a>) -> Self {
+impl Img {
+    pub fn new(buf: ImageBuffer<Rgba<u8>, Vec<u8>>, rect: Rect, style: Style) -> Self {
         Img { buf, rect, style }
     }
 }
@@ -72,37 +72,49 @@ impl Line {
     }
 }
 
-#[derive(Debug)]
-pub struct Text<'a> {
-    pub(super) text: String,
-    pub(super) total_height: u32,
-    pub(super) lines: Vec<Line>,
-    pub(super) style: Style<'a>,
-    pub(super) font: Font<'a>,
-    pub(super) max_line_height: f32,
-    pub(super) max_line_width: f32,
-    pub(super) textarea: TextArea<'a>,
+#[derive(Debug, Default)]
+pub struct LineMetrics {
+    pub total_height: u32,
+    pub max_line_height: f32,
+    pub max_line_width: f32,
 }
 
-impl<'a> Text<'a> {
+impl LineMetrics {
+    pub fn new(total_height: u32, max_line_height: f32, max_line_width: f32) -> Self {
+        LineMetrics {
+            total_height,
+            max_line_height,
+            max_line_width,
+        }
+    }
+}
+
+#[derive(Debug)]
+pub struct Text {
+    pub(super) text: String,
+    pub(super) metrics: LineMetrics,
+    pub(super) lines: Vec<Line>,
+    pub(super) style: Style,
+    // TODO: optimize static lifetime
+    pub(super) font: Font<'static>,
+    pub(super) textarea: TextArea,
+}
+
+impl Text {
     pub fn new(
         text: String,
         lines: Vec<Line>,
-        total_height: u32,
-        style: Style<'a>,
-        font: Font<'a>,
-        max_line_height: f32,
-        max_line_width: f32,
-        textarea: TextArea<'a>,
+        metrics: LineMetrics,
+        style: Style,
+        font: Font<'static>,
+        textarea: TextArea,
     ) -> Self {
         Text {
             text,
             lines,
-            total_height,
+            metrics,
             style,
             font,
-            max_line_height,
-            max_line_width,
             textarea,
         }
     }
