@@ -1,5 +1,5 @@
 use super::textarea::TextArea;
-use crate::element::{Element, Line, Rect, Text};
+use crate::element::{Element, Line, LineMetrics, Rect, Text};
 use crate::line_breaker::LineBreaker;
 use crate::style::{FlexDirection, Margin, Position, Style, TextOverflow};
 use crate::writer::OGImageWriter;
@@ -8,20 +8,15 @@ use rusttype::Font;
 use std::cell::RefCell;
 use std::str;
 
-impl<'a> OGImageWriter<'a> {
+impl OGImageWriter {
     pub(crate) fn process_text(
         &mut self,
-        textarea: RefCell<TextArea<'a>>,
+        textarea: RefCell<TextArea>,
         // Parent style that effect child element
-        style: Style<'a>,
+        style: Style,
         // Parent font that effect child element
-        font: Vec<u8>,
+        font: Font<'static>,
     ) -> Result<(), Error> {
-        let font = match Font::try_from_vec(font) {
-            Some(font) => font,
-            None => return Err(Error::InvalidFontBytes),
-        };
-
         let window_width = self.window.width as f32;
 
         let Margin(margin_top, margin_right, margin_bottom, margin_left) = style.margin;
@@ -116,11 +111,9 @@ impl<'a> OGImageWriter<'a> {
         let text_elm = Element::Text(Some(Text::new(
             text,
             lines,
-            total_height as u32,
+            LineMetrics::new(total_height as u32, max_line_height, max_line_width),
             style,
             font,
-            max_line_height,
-            max_line_width,
             textarea.into_inner(),
         )));
 
@@ -138,7 +131,7 @@ impl<'a> OGImageWriter<'a> {
             }
         }
 
-        self.tree.push(text_elm);
+        self.tree.0.push(text_elm);
 
         Ok(())
     }
@@ -151,7 +144,7 @@ impl<'a> OGImageWriter<'a> {
         font: &Font,
         textarea: &mut TextArea,
     ) -> Result<String, Error> {
-        let ellipsis = match style.text_overflow {
+        let ellipsis = match &style.text_overflow {
             TextOverflow::Ellipsis => "...",
             TextOverflow::Content(s) => s,
             TextOverflow::Clip => return Ok(text.to_string()),
@@ -193,7 +186,8 @@ impl<'a> OGImageWriter<'a> {
                 if split_text.range.start <= split_index && split_index <= split_text.range.end {
                     let end = split_text.range.end - split_index;
                     split_text.range.end -= end;
-                    split_text.text = &split_text.text[0..split_text.text.len() - end];
+                    split_text.text =
+                        (&split_text.text[0..split_text.text.len() - end]).to_string();
                     textarea.0.push(split_text);
                     break;
                 }
